from PySide6.QtWidgets import QLineEdit, QMainWindow
from PySide6.QtCore import Qt, QEvent
from ui.test_interface import Ui_MainWindow


class KeyboardTestWindow(QMainWindow):
    def __init__(self):
        super().__init__()

        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        self.lineEdit_list = self.ui.scrollAreaWidgetContents.findChildren(QLineEdit)

        self.ui.widget.set_keyboard_background("")

        self.ui.widget.hide()

        for line in self.lineEdit_list:
            line.set_keyboard_widget(self.ui.widget)
            line.set_scroll_area(self.ui.scrollArea)



from PySide6.QtGui import QFocusEvent, QMouseEvent
from PySide6.QtWidgets import QLineEdit, QWidget, QScrollArea
from PySide6.QtCore import Qt, Slot, QPoint, QElapsedTimer
from PySide6VirtualKeyboard.keyboard.virtual_keyboard import VirtualKeyboard


class VirtualKeyboardLineEdit(QLineEdit):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setFocusPolicy(Qt.FocusPolicy.ClickFocus)

        self.__parent = parent
        print(f"parent {self.__parent}")
        self.__keyboard = None

        self.__scroll_area = None

        self.__is_editing_finished = True

        self.__parent.installEventFilter(self)

        self.__keyboard_hide_timer = QElapsedTimer()

    def set_keyboard_widget(self, keyboard: VirtualKeyboard):
        self.__keyboard = keyboard
        self.__keyboard.return_pressed.connect(self.return_pressed_handler)

    def set_scroll_area(self, scroll_area: QScrollArea):
        self.__scroll_area = scroll_area

    def return_pressed_handler(self):
        self.__keyboard.hide()
        self.clearFocus()

    def focusInEvent(self, event):
        if self.__keyboard:
            self.__keyboard.show()
            self.__keyboard.set_input_line_edit(self)
            self.__is_editing_finished = False

            current_pos = QPoint(self.pos().x(), self.pos().y())
            print(f"current_pos {current_pos}")
            print(f"focus in event : {self.objectName()}")
            self.__scroll_area.verticalScrollBar().setValue(self.pos().y() - 50)

        super().focusInEvent(event)

    def focusOutEvent(self, event: QFocusEvent):

        self.__keyboard.hide()
        self.__is_editing_finished = True
        self.clearFocus()
        print(f"is editing finished: {self.__is_editing_finished}")

        super().focusOutEvent(event)

    def eventFilter(self, watched, event):
        if event.type() == QMouseEvent.Type.MouseButtonPress:
            if not self.__keyboard.geometry().contains(event.globalPosition().toPoint()):
                print(f"klavye dışına tıklandı: {self.objectName()} ")

                self.__keyboard_hide_timer.start()

        if event.type() == QMouseEvent.Type.MouseButtonRelease:
            if self.__keyboard_hide_timer.isValid() and self.__keyboard_hide_timer.elapsed() < 250:
                print("keyboard hide event")
                self.__is_editing_finished = True
                self.__keyboard.hide()
                self.clearFocus()
                self.__keyboard_hide_timer.invalidate()
        return super().eventFilter(watched, event)



import sys

from PySide6.QtWidgets import QApplication

from test_ui import KeyboardTestWindow


def main():
    app = QApplication(sys.argv)
    app.setApplicationName("PySide6VirtualKeyboard v0.1.0")
    app_window = KeyboardTestWindow()
    app_window.show()
    sys.exit(app.exec())


if __name__ == '__main__':
    main()



import sys
from PySide6.QtWidgets import QApplication, QMainWindow, QScrollArea, QLabel, QVBoxLayout, QWidget
from PySide6.QtCore import Qt, QPoint
from PySide6.QtGui import QMouseEvent


class VirtualKeyboardScrollArea(QScrollArea):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setWidgetResizable(True)
        content_widget = QWidget()
        self.setWidget(content_widget)

        self.setFocusPolicy(Qt.FocusPolicy.NoFocus)

        self.start_pos = None
        self.vertical_scroll_start_value = 0

        self.verticalScrollBar().setVisible(False)

    def mousePressEvent(self, event):
        if event.button() == Qt.MouseButton.LeftButton:
            self.start_pos = event.position().toPoint()
            self.vertical_scroll_start_value = self.verticalScrollBar().value()
        super().mousePressEvent(event)

    def mouseMoveEvent(self, event):
        if self.start_pos is not None:
            delta = event.position().toPoint() - self.start_pos
            new_value = self.vertical_scroll_start_value - delta.y()
            self.verticalScrollBar().setValue(new_value)
        super().mouseMoveEvent(event)

    def mouseReleaseEvent(self, event):
        if event.button() == Qt.MouseButton.LeftButton:
            self.start_pos = None
        super().mouseReleaseEvent(event)


class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()
        self.setWindowTitle("Mouse Event Scroll Example")
        self.resize(400, 300)

        scroll_area = VirtualKeyboardScrollArea()
        self.setCentralWidget(scroll_area)


if __name__ == "__main__":
    app = QApplication(sys.argv)
    window = MainWindow()
    window.show()
    sys.exit(app.exec())



from PySide6.QtWidgets import QWidget, QPushButton, QLineEdit
from PySide6.QtCore import Qt, Signal, Slot

from PySide6VirtualKeyboard.ui.interface import Ui_KeyboardUI


# import PySide6VirtualKeyboard.resources.resources_rc


def to_lower_turkish(text):
    # Türkçe karakterler için küçük harf dönüşüm tablosu
    translation_table = str.maketrans("ABCÇDEFGĞHIİJKLMNOÖPRSŞTUÜVYZQWX",
                                      "abcçdefgğhıijklmnoöprsştuüvyzqwx")
    return text.translate(translation_table)


def to_upper_turkish(text):
    # Türkçe karakterler için büyük harf dönüşüm tablosu
    translation_table = str.maketrans("abcçdefgğhıijklmnoöprsştuüvyzqwx",
                                      "ABCÇDEFGĞHIİJKLMNOÖPRSŞTUÜVYZQWX")
    return text.translate(translation_table)


class VirtualKeyboard(QWidget):
    return_pressed = Signal()

    def __init__(self, parent=None):
        super().__init__(parent)

        self.__ui = Ui_KeyboardUI()
        self.__ui.setupUi(self)
        self.__ui.retranslateUi(self)

        self.setWindowFlag(Qt.WindowType.FramelessWindowHint)
        self.setWindowModality(Qt.WindowModality.ApplicationModal)

        # Ui elements
        self.__input_line_edit = None

        self.__shift_button = self.__ui.pushButton_shift
        self.__backspace_button_abc = self.__ui.pushButton_abc_backspace

        self.__stacked_widget = self.__ui.stackedWidget_keyboard_layout

        self.__numeric_button_list_abc = self.__ui.widget_numeric_button_abc.findChildren(QPushButton)
        self.__numeric_button_list_123 = self.__ui.widget_numeric_button_123.findChildren(QPushButton)

        self.__symbol_button_list_123 = self.__ui.widget_123_1.findChildren(QPushButton) + \
                                        self.__ui.widget_123_2.findChildren(QPushButton)

        self.__symbol_button_list_sym = self.__ui.widget_sym_1.findChildren(QPushButton) + \
                                        self.__ui.widget_sym_2.findChildren(QPushButton) + \
                                        self.__ui.widget_sym_3.findChildren(QPushButton) + \
                                        self.__ui.widget_sym_4.findChildren(QPushButton)

        self.__letter_button_list = self.__ui.widget_abc_1.findChildren(QPushButton) + \
                                    self.__ui.widget_abc_2.findChildren(QPushButton) + \
                                    self.__ui.widget_abc_3.findChildren(QPushButton)

        self.__comma_button_list = [self.__ui.pushButton_abc_comma, self.__ui.pushButton_123_comma,
                                    self.__ui.pushButton_sym_comma]

        self.__dot_button_list = [self.__ui.pushButton_abc_dot, self.__ui.pushButton_123_dot,
                                  self.__ui.pushButton_sym_dot]

        # Init functions
        self.signal_connect()

    def set_keyboard_background(self, stylesheet):
        self.__ui.frame_background.setStyleSheet(stylesheet)

    def set_space_buttons_image(self, image_path):
        style_sheet = f"""
        QPushButton{{
            background-color: rgba(71, 68, 68, 209);
            padding-top: 60px;
            image: url({image_path});
            border-radius: 15px;
        }}
        QPushButton:pressed {{
            background-color: rgba(50, 50, 50, 209);
        }}
        """
        self.__ui.pushButton_abc_space.setStyleSheet(style_sheet)
        self.__ui.pushButton_123_space.setStyleSheet(style_sheet)
        self.__ui.pushButton_sym_space.setStyleSheet(style_sheet)

    def signal_connect(self):

        self.__ui.pushButton_abc_to_123.clicked.connect(lambda: self.__stacked_widget.setCurrentIndex(1))
        self.__ui.pushButton_123_to_symbols.clicked.connect(lambda: self.__stacked_widget.setCurrentIndex(2))
        self.__ui.pushButton_123_to_abc.clicked.connect(lambda: self.__stacked_widget.setCurrentIndex(0))
        self.__ui.pushButton_sym_to_abc.clicked.connect(lambda: self.__stacked_widget.setCurrentIndex(0))
        self.__ui.pushButton_sym_to_123.clicked.connect(lambda: self.__stacked_widget.setCurrentIndex(1))

        self.__shift_button.clicked.connect(self.shift_button_handler)

        self.connect_buttons(self.__letter_button_list)
        self.connect_buttons(self.__numeric_button_list_abc)
        self.connect_buttons(self.__numeric_button_list_123)
        self.connect_buttons(self.__symbol_button_list_123)
        self.connect_buttons(self.__symbol_button_list_sym)
        self.connect_buttons(self.__comma_button_list)
        self.connect_buttons(self.__dot_button_list)

        self.__ui.pushButton_abc_backspace.clicked.connect(lambda: self.__input_line_edit.backspace())
        self.__ui.pushButton_123_backspace.clicked.connect(lambda: self.__input_line_edit.backspace())
        self.__ui.pushButton_sym_backspace.clicked.connect(lambda: self.__input_line_edit.backspace())

        self.__ui.pushButton_abc_enter.clicked.connect(lambda: self.return_pressed.emit())
        self.__ui.pushButton_123_enter.clicked.connect(lambda: self.return_pressed.emit())
        self.__ui.pushButton_sym_enter.clicked.connect(lambda: self.return_pressed.emit())

        self.__ui.pushButton_abc_space.clicked.connect(lambda: self.insert_text(" "))
        self.__ui.pushButton_123_space.clicked.connect(lambda: self.insert_text(" "))
        self.__ui.pushButton_sym_space.clicked.connect(lambda: self.insert_text(" "))

    def set_input_line_edit(self, input_line: QLineEdit):
        self.__input_line_edit = input_line
        print(f"input_line: {input_line.objectName()}")

    def connect_buttons(self, button_list):
        for button in button_list:
            if len(button.text()) == 1:
                print(f"button text: {button.text()}")
                button.clicked.connect(lambda checked, text=button.text(): self.insert_text(text))

    @Slot()
    def shift_button_handler(self):
        if self.__shift_button.isChecked():
            for button in self.__letter_button_list:
                if button.text() != "":
                    button.setText(to_lower_turkish(button.text()))
        else:
            for button in self.__letter_button_list:
                if button.text() != "":
                    button.setText(to_upper_turkish(button.text()))

    @Slot()
    def insert_text(self, text):
        print(f"insert text: {text}")
        cursor = self.__input_line_edit.cursorPosition()
        self.__input_line_edit.insert(text)
        self.__input_line_edit.setCursorPosition(cursor + 1)

    @Slot()
    def hide(self):
        self.__ui.stackedWidget_keyboard_layout.setCurrentIndex(0)
        super().hide()
